/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2007
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	Serial_user.c
**  Abstract :	This file implements device driver for Serial module.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.10 [6 Mar. 2007]
**
**  Device :	uPD78F0533_64
**
**  Compiler :	CC78K0
**
**  Creation date:	2007/04/09
**  
*******************************************************************************
*/

#pragma interrupt INTIIC0 MD_INTIIC0
/*
*******************************************************************************
** Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Serial.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
extern	volatile UCHAR  gIic0MasterStatusFlag; 
extern	volatile UCHAR	gIic0SlaveStatusFlag;
extern	volatile USHORT gIic0TxCnt;
extern	volatile UCHAR* gpIic0TxAddress;
extern	volatile UCHAR* gpIic0RxAddress;
extern	volatile USHORT gIic0RxCnt;
extern	volatile USHORT gIic0RxLen;
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */


/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is INTIIC0 interrupt service routine.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
__interrupt void MD_INTIIC0( void )
{
	MD_STATUS sta;
	if( IICS0 & IIC0_STATUS_MASTER ) {
		sta = IIC0_MasterHandler();
	}
}

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is iic slave handler.
**
**  Parameters:
**	None
**
**  Returns:
**	MD_OK/MD_SLAVE_SEND_END/MD_SLAVE_RCV_END
**	MD_ERROR/MD_SPT/MD_NACK
**
**-----------------------------------------------------------------------------
*/
MD_STATUS IIC0_SlaveHandler( void )
{
	/* control for stop condition */
	if( SPD0 == 1 ){    
		/* get stop condition */
		SPIE0 = 0;
		gIic0SlaveStatusFlag = 1;
		return MD_OK;
	}

	if( !(gIic0SlaveStatusFlag & ADDRESS_COMPLETE) )
	{
		if(COI0 == 1)
		{
			SPIE0 = 1;    
			gIic0SlaveStatusFlag |= ADDRESS_COMPLETE;
			if(TRC0 == 1)
			{
				WTIM0 = 1;
			}
			else
			{
				ACKE0 = 1;
				WTIM0 = 0;
				WREL0 = 1;
				return MD_OK;
			}
		}
		else{
			return MD_ERROR;
		}
	}

	if(TRC0 == 1){
		if(ACKD0 == 0){
			return MD_NACK;
		}
		if( gIic0TxCnt > 0 ){
			IIC0 = *gpIic0TxAddress;
			gpIic0TxAddress++;
			gIic0TxCnt--;
		}
		else{
			WREL0 = 1;
			return MD_SLAVE_SEND_END;
		}
	}
	else{
		if( gIic0RxCnt < gIic0RxLen )
		{
			*gpIic0RxAddress = IIC0;
			gpIic0RxAddress++;
			gIic0RxCnt++;
			if(gIic0RxCnt == gIic0RxLen)
			{
				ACKE0 = 0;
				WTIM0 = 1;
				WREL0 = 1;
				return MD_SLAVE_RCV_END;
			}
		}
		WREL0 = 1;
	}
	return MD_OK;
}

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function is iic master handler.
**
**  Parameters:
**	None
**
**  Returns:
**	MD_OK/MD_MASTER_SEND_END/MD_MASTER_RCV_END
**	MD_ERROR/MD_SPT/MD_NACK
**
**-----------------------------------------------------------------------------
*/
MD_STATUS IIC0_MasterHandler( void )
{
	/* control for communication */
	if( IICBSY == 0 && gIic0TxCnt != 0 ){
		return MD_SPT;
	}

	/* control for sended address */
	if( !(gIic0MasterStatusFlag & ADDRESS_COMPLETE) )
	{
		if( ACKD0 == 1)
		{
			gIic0MasterStatusFlag |= ADDRESS_COMPLETE;	/* address complete */
			if(TRC0 == 1)
			{
				WTIM0 = 1;
			}
			else
			{
				ACKE0 = 1;
				WTIM0 = 0;
				WREL0 = 1;
				return MD_OK;
			}
		}
		else
		{
			return MD_NACK;
		}
	}

	/* master send control */
	if(TRC0 == 1)
	{
		if( ACKD0 == 0 && gIic0TxCnt != 0 )
		{
			return MD_NACK;
		}
		if( gIic0TxCnt > 0 )
		{
			IIC0 = *gpIic0TxAddress;
			gpIic0TxAddress++;
			gIic0TxCnt--;
		}
		else
		{
			SPT0 = 1;
			return MD_MASTER_SEND_END;
		}
	}

	/* master receive control */
	else
	{
		if( gIic0RxCnt < gIic0RxLen )
		{
			*gpIic0RxAddress = IIC0;
			gpIic0RxAddress++;
			gIic0RxCnt++;
			if(gIic0RxCnt == gIic0RxLen)
			{
				ACKE0 = 0;
				WREL0 = 1;

				WTIM0=1;
				SPT0 = 1;
				return MD_MASTER_RCV_END;
			}
		}
		WREL0 = 1;
	}
	return MD_OK;
}

/*
**-----------------------------------------------------------------------------
**
**  Abstract:
**	This function callback function open for users operation when IIC0 master find slave.
**
**  Parameters:
**	None
**
**  Returns:
**	None
**
**-----------------------------------------------------------------------------
*/
void CALL_IIC0_MasterFindSlave( void )
{
	/* Start user code. Do not edit comment generated here */
	/* TODO. Add user defined callback routine */
	/* End user code. Do not edit comment generated here */
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */


